@@ -17,6 +17,8 @@ namespace Tomboy
 		AddinManager addin_mgr;
 		TrieController trie_controller;
 
+		public static string NoteTemplateTitle = Catalog.GetString ("New Note Template");
+
 		static string start_note_uri = String.Empty;
 
 		static NoteManager ()
@@ -66,7 +68,8 @@ public NoteManager (string directory) :
 
 			Tomboy.ExitingEvent += OnExitingEvent;
 		}
-
+		
+		
 		// Create the TrieController. For overriding in test methods.
 		protected virtual TrieController CreateTrieController ()
 		{
@@ -112,12 +115,14 @@ public NoteManager (string directory) :
 		{
 			if (NoteRenamed != null)
 				NoteRenamed (note, old_title);
+			this.notes.Sort (new CompareDates ());
 		}
 
 		void OnNoteSave (Note note)
 		{
 			if (NoteSaved != null)
 				NoteSaved (note);
+			this.notes.Sort (new CompareDates ());
 		}
 
 		protected virtual void CreateStartNotes ()
@@ -162,28 +167,17 @@ public NoteManager (string directory) :
 			                           "Also, if you type the name of another note in your " +
 			                           "current note, it will automatically be linked for you." +
 			                           "</note-content>");
-			
-			string tags_note_content = Catalog.GetString ("<note-content>"+
-			                                              "Tagging In Tomboy\n\n"+
-			                                              "Notes can be tagged with the tag bar "+
-			                                              "at the bottom of the note screen. "+
-			                                              "Pressing <bold>Tab</bold> in the tag bar will "+
-			                                              "autocomplete the nearest tag. To confirm the displayed tag "+
-			                                              "press <bold>Enter</bold>."+
-			                                              "</note-content>");
 
 			try {
 				Note start_note = Create (Catalog.GetString ("Start Here"),
 				                          start_note_content);
-				start_note.QueueSave (true);
+				start_note.QueueSave (ChangeType.ContentChanged);
 				Preferences.Set (Preferences.START_NOTE_URI, start_note.Uri);
 
 				Note links_note = Create (Catalog.GetString ("Using Links in Tomboy"),
 				                          links_note_content);
-				links_note.QueueSave (true);
-				
-				Note tag_note = Create (Catalog.GetString ("Tags in Tomboy"),tags_note_content);
-				tag_note.QueueSave(true);
+				links_note.QueueSave (ChangeType.ContentChanged);
+
 				start_note.Window.Show ();
 			} catch (Exception e) {
 				Logger.Warn ("Error creating start notes: {0}\n{1}",
@@ -229,7 +223,7 @@ public NoteManager (string directory) :
 						note.Window.Show ();
 
 					note.IsOpenOnStartup = false;
-					note.QueueSave (false);
+					note.QueueSave (ChangeType.NoChange);
 				}
 			}
 
@@ -257,8 +251,11 @@ public NoteManager (string directory) :
 			}
 
 			Logger.Log ("Saving unsaved notes...");
-
-			foreach (Note note in notes) {
+			
+			// Use a copy of the notes to prevent bug #510442 (crash on exit
+			// when iterating the notes to save them.
+			List<Note> notesCopy = new List<Note> (notes);
+			foreach (Note note in notesCopy) {
 				// If the note is visible, it will be shown automatically on
 				// next startup
 				if (note.HasWindow && note.Window.Visible)
@@ -363,7 +360,9 @@ public NoteManager (string directory) :
 		}
 
 		// Create a new note with the specified title, and a simple
-		// "Describe..." body which will be selected for easy overwrite.
+		// "Describe..." body or the body from the "New Note Template"
+		// note if it exists.  If the "New Note Template" body is found
+		// the text will not automatically be highlighted.
 		private Note CreateNewNote (string title, string guid)
 		{
 			string body = null;
@@ -371,9 +370,18 @@ public NoteManager (string directory) :
 			title = SplitTitleFromContent (title, out body);
 			if (title == null)
 				return null;
-
-			if (body == null)
-				body = Catalog.GetString ("Describe your new note here.");
+			
+			Note note_template = Find (NoteTemplateTitle);
+			if (note_template != null) {
+				// Use the body from the "New Note Template" note
+				string xml_content =
+					note_template.XmlContent.Replace (NoteTemplateTitle, title);
+				return CreateNewNote (title, xml_content, guid);
+			}
+			
+			// Use a simple "Describe..." body and highlight
+			// it so it can be easily overwritten
+			body = Catalog.GetString ("Describe your new note here.");
 
 			string header = title + "\n\n";
 			string content =
@@ -423,6 +431,50 @@ public NoteManager (string directory) :
 
 			return new_note;
 		}
+		
+		/// <summary>
+		/// Get the existing template note or create a new one
+		/// if it doesn't already exist.
+		/// </summary>
+		/// <returns>
+		/// A <see cref="Note"/>
+		/// </returns>
+		public Note GetOrCreateTemplateNote ()
+		{
+			Note template_note = Find (NoteTemplateTitle);
+			if (template_note == null) {
+				template_note =
+					Create (NoteTemplateTitle,
+							GetNoteTemplateContent (NoteTemplateTitle));
+					
+				// Select the initial text
+				NoteBuffer buffer = template_note.Buffer;
+				Gtk.TextIter iter = buffer.GetIterAtLineOffset (2, 0);
+				buffer.MoveMark (buffer.SelectionBound, iter);
+				buffer.MoveMark (buffer.InsertMark, buffer.EndIter);
+
+				// Flag this as a template note
+				Tag tag = TagManager.GetOrCreateSystemTag (TagManager.TemplateNoteSystemTag);
+				template_note.AddTag (tag);
+
+				template_note.QueueSave (ChangeType.ContentChanged);
+			}
+			
+			return template_note;
+		}
+		
+		public static string GetNoteTemplateContent (string title)
+		{
+			const string base_xml =
+			        "<note-content>" +
+			        "<note-title>{0}</note-title>\n\n" +
+			        "{1}" +
+			        "</note-content>";
+
+			return string.Format (base_xml,
+			                      XmlEncoder.Encode (title),
+			                      Catalog.GetString ("Describe your new note here."));
+		}
 
 		public Note Find (string linked_title)
 		{
@@ -467,7 +519,7 @@ public NoteManager (string directory) :
 			get {
 				// FIXME: Only sort on change by listening to
 				//        Note.Saved or Note.Buffer.Changed
-				notes.Sort (new CompareDates ());
+				//notes.Sort (new CompareDates ());
 				return notes;
 			}
 		}
